# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

import os.path
import sys
import wasp.demo

from StringIO import StringIO
from unittest import TestCase,TestSuite,makeSuite
from wasp import Message,Notification,status
from wasp.demo import SendMessage,ProcessResponse
from wasp.tests import BaseSendTests, BaseProcessResponse, setUp, FunctionalLayer
from textwrap import dedent
from zope.app.testing.functional import ZCMLLayer
from zope.app.testing.functional import FunctionalDocFileSuite
from zope.component.testing import tearDown as componentTearDown
from zope.testing import doctest

class Checker:

    def __init__(self,u):
        self.u = u
        self.o = StringIO()

    def __call__(self,*args,**kw):
        try:
            sys.stdout = self.o
            return self.u(*args,**kw)
        finally:
            sys.stdout = sys.__stdout__

    def check(self,t,expected):
        t.assertEqual(self.o.getvalue(),dedent(expected))
            
class SendTests(BaseSendTests):

    klass = SendMessage

    def test_send(self):
        c = Checker(self.u)
        self.assertEqual(c('123','test message','456'),self.response)
        c.check(self,"""\
        Send to: '123'
        Message: 'test message'
             Id: '456'
        """)

    def test_send_no_id(self):
        c = Checker(self.u)
        self.assertEqual(c('123','test message'),self.response)
        c.check(self,"""\
        Send to: '123'
        Message: 'test message'
             Id: None
        """)

    def test_send_false_response(self):
        c = Checker(SendMessage(response=False))
        self.assertEqual(c('123','test message'),False)
        c.check(self,"""\
        Send to: '123'
        Message: 'test message'
             Id: None
        """)
        
class ProcessResponseTests(BaseProcessResponse):

    klass = ProcessResponse

    def test_message(self):
        self.cmp(
            self.u.process(None,{
            'type':'message',
            'msisdn':'1234',
            'message_text':'message',
            },''),
            (
            Message(
                '1234',
                'message'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_message_missingparams(self):
        self.cmp(
            self.u.process(None,{
            'type':'message',
            },''),
            (
            Message(
                '<not supplied>',
                '<not supplied>',
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_notification(self):
        self.cmp(
            self.u.process(None,{
            'type':'notification',
            'message_id':'1234',
            'status':'failed',
            'details':'it sucked',
            },''),
            (
            Notification(
                '1234',
                status.failed,
                'it sucked'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_notification_rubbish_status(self):
        self.cmp(
            self.u.process(None,{
            'type':'notification',
            'message_id':'1234',
            'status':'moo',
            'details':'it sucked',
            },''),
            (
            Notification(
                '1234',
                status.delivered,
                'it sucked'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_notification_missingparams(self):
        self.cmp(
            self.u.process(None,{
            'type':'notification',
            },''),
            (
            Notification(
                '<not supplied>',
                status.delivered,
                '<not supplied>',
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_nothing(self):
        self.cmp(
            self.u.process(None,{},''),
            (),
            )
        self.assertEqual(self.u.response(),'done')

    def test_no_type(self):
        self.cmp(
            self.u.process(None,{
            'message_id':'1234',
            'status':'failed',
            'detailed':'it sucked',
            },''),
            ()
            )
        self.assertEqual(self.u.response(),'done')

    def test_rubbish_type(self):
        self.cmp(
            self.u.process(None,{
            'type':'fish',
            },''),
            ()
            )
        self.assertEqual(self.u.response(),'done')

FunctionalLayer = ZCMLLayer(
    os.path.join(os.path.split(__file__)[0], '..','ftesting.zcml'),
    __name__,
    'wasp.demo Functional Layer',
    allow_teardown=True)

def test_suite():
    
    doc_suite = FunctionalDocFileSuite(
            'readme.txt',
            setUp=setUp,
            tearDown=componentTearDown,
            optionflags=doctest.REPORT_NDIFF|doctest.ELLIPSIS
            )
    doc_suite.layer = FunctionalLayer

    return TestSuite((
        doc_suite,
        makeSuite(SendTests),
        makeSuite(ProcessResponseTests),
        ))



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

import os.path
import sys
import urllib2
import wasp.bulksms

from StringIO import StringIO
from unittest import TestCase,TestSuite,makeSuite
from wasp import Message,Notification,status,SendException
from wasp.interfaces import ISendMessage
from wasp.bulksms import SendMessage,ProcessResponse
from wasp.tests import BaseSendTests, BaseProcessResponse, setUp, FunctionalLayer
from zope.app.appsetup.appsetup import getConfigContext
from zope.app.testing.functional import ZCMLLayer
from zope.app.testing.functional import FunctionalDocFileSuite
from zope.component import getUtility
from zope.component.testing import tearDown as componentTearDown
from zope.configuration import xmlconfig
from zope.security.interfaces import Unauthorized
from zope.testing import doctest

class Response(StringIO):

    def __init__(self,parent,url,data,response):
        StringIO.__init__(self,response)
        self.url = url
        self.data = data
        self.parent = parent

    def read(self):
        if self.parent:
            self.parent.opened = self.url
        else:
            print 'opened:',repr(self.url)
        if self.data:
            if self.parent:
                self.parent.data = self.data
            else:
                print 'data:',repr(self.data)
        return StringIO.read(self)
        
class TestOpenerDirector:

    response = ''
    opened = ''
    data = ''
    
    def __init__(self,store=False):
        self.store = store

    def open(self,fullurl,data=None):
        return Response(self.store and self,
                        fullurl,data,self.response)
    
    def set_response(self,response):
        self.response = response
        
class SendTests(BaseSendTests):

    klass = SendMessage

    response = False
    
    def setUp(self):
        self.u = self.klass('testusername','testpassword')
        self.opener = TestOpenerDirector(store=True)
        self.original_opener = urllib2._opener
        urllib2.install_opener(self.opener)
        self.opener.set_response('0|IN_PROGRESS|71070172')

    def tearDown(self):
        urllib2.install_opener(self.original_opener)
        
    def test_send(self):
        self.assertEqual(self.u('123','test message','456'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=test%20message&msisdn=123&want_report=0&source_id=456'
            )

    def test_send_no_id(self):
        self.assertEqual(self.u('123','test message'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=test%20message&msisdn=123&want_report=0'
            )

    def test_error_on_send(self):
        self.opener.set_response('22|INTERNAL_FATAL_ERROR|71070172')
        self.assertRaises(SendException,self.u,'123','test message')
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=test%20message&msisdn=123&want_report=0'
            )
        
    def test_secure(self):
        self.u = self.klass('testusername','testpassword',secure=True)
        self.assertEqual(self.u('123','test message'),self.response)
        self.assertEqual(
            self.opener.opened,
            'https://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=test%20message&msisdn=123&want_report=0'
            )
        
    def test_want_report(self):
        self.u = self.klass('testusername','testpassword',want_report=True)
        self.assertEqual(self.u('123','test message'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=test%20message&msisdn=123&want_report=1'
            )

    def test_quote_msisdn(self):
        self.assertEqual(self.u('1%3','message','456'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=message&msisdn=1%253&want_report=0&source_id=456'
            )

    def test_quote_message(self):
        self.assertEqual(self.u('123','test message &fail=1','456'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=test%20message%20%26fail%3D1&msisdn=123&want_report=0&source_id=456'
            )

    def test_quote_message_id(self):
        self.assertEqual(self.u('123','message','%456'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=testpassword&message=message&msisdn=123&want_report=0&source_id=%25456'
            )

    def test_quote_username(self):
        self.u = self.klass('user&name','testpassword')
        self.assertEqual(self.u('123','message','456'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=user%26name&password=testpassword&message=message&msisdn=123&want_report=0&source_id=456'
            )
        
    def test_quote_password(self):
        self.u = self.klass('testusername','pass&word')
        self.assertEqual(self.u('123','message','456'),self.response)
        self.assertEqual(
            self.opener.opened,
            'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0?username=testusername&password=pass%26word&message=message&msisdn=123&want_report=0&source_id=456'
            )

class ProcessResponseTests(BaseProcessResponse):

    def setUp(self):
        self.u=ProcessResponse('password')

    def test_message(self):
        self.cmp(
            self.u.process(None,{
            'pass':'password',
            'type':'message',
            'msisdn':'1234',
            'sender':'345',
            'msg_class':'2',
            'dca':'7bit',
            'msg_id':'40436130',
            'message':'message',
            'origin_id':'id123',
            'source_id':'id456',
            'network_id':'65501',
            'referring_msg_id':'99172705',
            'referring_batch_id':'71070172',
            },''),
            (
            Message(
                '345',
                'message'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_message_missingparams(self):
        self.cmp(
            self.u.process(None,{
            'pass':'password',
            'type':'message',
            },''),
            (
            Message(
                '<not supplied>',
                '<not supplied>',
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_notification(self):
        self.cmp(
            self.u.process(None,{
            'pass':'password',
            'type':'notification',
            'msg_id':'9916694',
            'msisdn':'123',
            'status':'11',
            'batch_id':'71070172',
            'source_id':'1234',
            },''),
            (
            Notification(
                '1234',
                status.delivered,
                '11: Delivered to mobile'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_notification_rubbish_status(self):
        self.cmp(
            self.u.process(None,{
            'pass':'password',
            'type':'notification',
            'msg_id':'9916694',
            'msisdn':'123',
            'status':'moo',
            'batch_id':'71070172',
            'source_id':'1234',
            },''),
            (
            Notification(
                '1234',
                status.failed,
                '70: Unknown upstream status'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_message_id_fallback(self):
        self.cmp(
            self.u.process(None,{
            'pass':'password',
            'type':'notification',
            'msg_id':'9916694',
            'msisdn':'123',
            'status':'11',
            'batch_id':'71070172',
            },''),
            (
            Notification(
                '9916694',
                status.delivered,
                '11: Delivered to mobile'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')
        
    def test_notification_missingparams(self):
        self.cmp(
            self.u.process(None,{
            'pass':'password',
            'type':'notification',
            },''),
            (
            Notification(
                '<not supplied>',
                status.failed,
                '70: Unknown upstream status'
                ),
            )
            )
        self.assertEqual(self.u.response(),'done')

    def test_nothing(self):
        self.assertRaises(Unauthorized,self.u.process,None,{},'')

    def test_no_type(self):
        self.assertRaises(Exception,self.u.process,None,{'pass':'password'},'')

    def test_rubbish_type(self):
        self.assertRaises(Exception,self.u.process,None,{'pass':'password',
                                                         'type':'fish'},'')

    def test_wrong_password(self):
        self.assertRaises(Unauthorized,self.u.process,None,{'pass':'something'},'')

class TestZCML(TestCase):

    # The bulk of the ZCML is tested in the doctests.
    # The tests here are for the rare bits that aren't.
    layer = ZCMLLayer(
        os.path.join(os.path.split(__file__)[0], '..','ftesting.zcml'),
        __name__,
        'wasp.bulksms.zcml Functional Layer',
        allow_teardown=True)

    def setUp(self):
        self.load_zcml('<include package="wasp.bulksms" file="meta.zcml" />')
        
    def load_zcml(self,text):
        config = getConfigContext()
        xmlconfig.string(text,config)
        
    def test_secure(self):
        self.load_zcml('''
 <configure 
     xmlns="http://namespaces.zope.org/zope"
     xmlns:wasp="http://namespaces.simplistix.co.uk/wasp">
     <wasp:sender 
         username="testuser"
         password="testpassword"
         secure="True"
     />
 </configure>
        ''')
        u = getUtility(ISendMessage)
        self.assertEqual(
            u.url,
            'https://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0?username=testuser&password=testpassword&message=%s&msisdn=%s&want_report=0'
            )

FunctionalLayer = ZCMLLayer(
    os.path.join(os.path.split(__file__)[0], '..','ftesting.zcml'),
    __name__,
    'wasp.bulksms Functional Layer',
    allow_teardown=True)

_original_opener = None

def urlSetUp(test):
    global _original_opener
    setUp(test)
    # *sulk* no api to retrieve this
    _original_opener = urllib2._opener
    # put in the test one
    opener = TestOpenerDirector()
    urllib2.install_opener(opener)
    test.globs['opener']=opener
    opener.set_response('0|IN_PROGRESS|71070172')
    
def urlTearDown(test):
    urllib2.install_opener(_original_opener)
    componentTearDown(test)
    
def test_suite():
    
    doc_suite = FunctionalDocFileSuite(
            'readme.txt',
            setUp=urlSetUp,
            tearDown=urlTearDown,
            optionflags=doctest.REPORT_NDIFF|doctest.ELLIPSIS
            )
    doc_suite.layer = FunctionalLayer

    return TestSuite((
        doc_suite,
        makeSuite(SendTests),
        makeSuite(ProcessResponseTests),
        makeSuite(TestZCML),
        ))



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

from wasp import Notification,Message,status
from wasp.interfaces import ISendMessage,IProcessResponse,INotifyMessage,IReceiveMessage
from zope.interface import implements

class SendMessage:

    implements(ISendMessage)

    def __init__(self,response=True):
        self.response = response
        
    def __call__(self,msisdn,message_text,message_id=None):
        print "Send to:",repr(msisdn)
        print "Message:",repr(message_text)
        print "     Id:",repr(message_id)
        return self.response
    
class EchoMessage:

    implements(IReceiveMessage)
    
    def __call__(self,msisdn,message_text):
        print "      msisdn:",repr(msisdn)
        print "message_text:",repr(message_text)

class EchoNotification:

    implements(INotifyMessage)
    
    def __call__(self,message_id,status,details):
        print "message_id:",repr(message_id)
        print "    status:",repr(status)
        print "   details:",repr(details)
        
class ProcessResponse:

    implements(IProcessResponse)
    
    def process(self,request,form,body):
        if form.get('type')=='message':
            yield Message(
                form.get('msisdn','<not supplied>'),
                form.get('message_text','<not supplied>'),
                )
        if form.get('type')=='notification':
            s = getattr(status,form.get('status','delivered'),status.delivered)
            yield Notification(
                form.get('message_id','<not supplied>'),
                s,
                form.get('details','<not supplied>'),
                )

    def response(self):
        return 'done'

class EchoResponse:

    implements(IProcessResponse)
    
    def process(self,request,form,body):
        print 'method:',request.method
        print '  form:',form
        print '  body:',repr(body)
        return ()
        
    def response(self):
        return 'done'



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

from wasp import Notification,Message,status,SendException
from wasp.interfaces import ISendMessage,IProcessResponse
from urllib import quote
from urllib2 import urlopen
from zope.interface import implements
from zope.security.interfaces import Unauthorized

status_mapping = {
    '0':'In progress',
    '1':'Scheduled',
    '11':'Delivered to mobile',
    '22':'Internal fatal error',
    '23':'Authentication failure',
    '24':'Data validation failed',
    '25':'You do not have sufficient credits',
    '26':'Upstream credits not available',
    '27':'You have exceeded your daily quota',
    '28':'Upstream quota exceeded',
    '29':'Message sending cancelled',
    '31':'Unroutable',
    '32':'Blocked',
    '33':'Failed: censored',
    '40':'Temporarily unavailable',
    '50':'Delivery failed',
    '51':'Delivery to phone failed',
    '52':'Delivery to network failed',
    '53':'Message expired',
    '54':'Failed on remote network',
    '56':'Failed: remotely censored',
    '57':'Failed due to fault on handset',
    '64':'Queued for retry after temporary failure delivering, due to fault on handset (transient)',
    '70':'Unknown upstream status',
    }

def get_message(code):
    code=str(code)
    if code not in status_mapping:
        code = '70'
    return '%s: %s'%(code,status_mapping[code])

class SendMessage:

    implements(ISendMessage)

    def __init__(self,username,password,secure=False,want_report=False):
        if secure:
            url = 'https://bulksms.2way.co.za/eapi/submission/send_sms/2/2.0'
        else:
            url = 'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0'
        url += ('?username=%s&password=%s&message=%%s&msisdn=%%s&want_report=%s' % (
            quote(username).replace('%','%%'),
            quote(password).replace('%','%%'),
            want_report and '1' or '0',
            ))
        self.url = str(url)
        
    def __call__(self,msisdn,message_text,message_id=None):
        url = self.url % (
            quote(message_text),
            quote(msisdn),
            )
        if message_id:
            url += ('&source_id='+quote(message_id))
        code,message,batch_id = urlopen(url).read().split('|')
        if code!='0':
            raise SendException(get_message(code))
        return False
    
class ProcessResponse:

    implements(IProcessResponse)

    def __init__(self,incoming_password):
        self.incoming_password = incoming_password
        
    def process(self,request,form,body):
        
        if form.get('pass')!=self.incoming_password:
            raise Unauthorized()

        type = form.get('type')

        if type=='message':
            return (Message(
                form.get('sender','<not supplied>'),
                form.get('message','<not supplied>'),
                ),)
        
        if type=='notification':
            code = form.get('status')
            if code=='11':
                s = status.delivered
            else:
                s = status.failed
            return (Notification(
                form.get('source_id',form.get('msg_id','<not supplied>')),
                s,
                form.get('details',get_message(code)),
                ),)

        raise Exception('Neither message nor notification was specified')

    def response(self):
        return 'done'




# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

from wasp.demo import SendMessage
from wasp.interfaces import ISendMessage
from zope.component.zcml import utility
from zope.configuration.fields import Bool
from zope.interface import Interface

class ISenderDirective(Interface):
    """Setup a demo SendMessage utility."""

    response = Bool(
        title=u"Return value",
        description=u"Configures whether the SendMessage utility will return True or False when called",
        required=False
        )

def setupUtility(_context,response=None):
    kw = {}
    if response is not None:
        kw['response']=response
    sender = SendMessage(**kw)
    utility(_context, ISendMessage, sender)



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.
import os.path

from StringIO import StringIO
from unittest import TestCase,TestSuite,makeSuite
from wasp import Message,Notification,Receiver
from wasp import send
from wasp.interfaces import ISendMessage,IProcessResponse,INotifyMessage,IReceiveMessage
from zope.app.appsetup.appsetup import getConfigContext
from zope.app.testing.functional import ZCMLLayer
from zope.app.testing.functional import FunctionalDocFileSuite
from zope.component import provideAdapter,provideUtility,ComponentLookupError
from zope.component.testing import tearDown as componentTearDown
from zope.configuration import xmlconfig
from zope.interface.verify import verifyObject
from zope.testbrowser.testing import Browser
from zope.testing import doctest

class BaseSendTests(TestCase):

    klass = None

    response = True
    
    def setUp(self):
        # This setup should be overridden if
        # more work needs to be done to set up the utility
        self.u = self.klass()
        
    def test_interface(self):
        verifyObject(ISendMessage, self.u)

    def test_send(self):
        # This should be overridden to actually
        # test the functionality of the utility.
        self.assertEqual(self.u('123','test message','456'),self.response)

    def test_send_no_id(self):
        # This should be overridden to actually
        # test the functionality of the utility.
        self.assertEqual(self.u('123','test message'),self.response)

    # subclasses should add further tests to test the functionality of
    # the particular implementation.
    
class BaseProcessResponse(TestCase):

    klass = None

    def setUp(self):
        # This setup should be overridden if
        # more work needs to be done to set up the utility
        self.u = self.klass()

    def cmp(self,actual,expected,u=None):
        actual = tuple(actual)
        self.assertEqual(
            len(expected),len(actual),
            "Expected %i results, %i returned" % (
            len(expected),len(actual),
            ))
        problems = []
        for a,e in zip(actual,expected):
            if a.__class__ is not e.__class__:
                problems.append(
                    "%r != %r" % (a.__class__,e.__class__)
                    )
                continue
            if a.__dict__!=e.__dict__:
                problems.append(
                    "%r != %r" % (a.__dict__,e.__dict__)
                    )
        if problems:
            self.fail('actual != expected:\n'+'\n'.join(problems))
        
    def test_interface(self):
        verifyObject(IProcessResponse, self.u)

    def test_message(self):
        # This should be overridden to actually
        # test the functionality of the utility.
        # (both the `process` and `response` methods!)
        raise NotImplementedError

    def test_notification(self):
        # This should be overridden to actually
        # test the functionality of the utility.
        # make sure Notification.status is in
        # wasp.status
        # (both the `process` and `response` methods!)
        raise NotImplementedError
    
    # subclasses should add further tests to test the functionality of
    # the particular implementation.
    
def test_import():
    """
    Mainly just as an example of a doc test ;-)
    
    >>> from wasp import send,Receiver,Message,Notification

    There's also a handy exception to raise when `wasp.send` fails:

    >>> from wasp import SendException
    >>> issubclass(SendException,Exception)
    True
    """

class Tests(TestCase):
    
    def test_statuses(self):
        # check that all statuses are subclasses of status
        import wasp.status
        for name in wasp.status.__dict__:
            i = name[0]
            if i=='_':
                continue
            obj = getattr(wasp.status,name)
            if i.isupper():
                self.failUnless(issubclass(obj,wasp.status.Status))
            else:    
                self.failUnless(isinstance(obj,wasp.status.Status))

    def test_message_interface(self):
        self.failUnless(Message.interface is IReceiveMessage)

    def test_message_normal(self):
        m = Message('123','some text')
        self.assertEqual((m.msisdn,m.message_text),('123','some text'))

    def test_message_keyword(self):
        m = Message(message_text='some text',msisdn='123')
        self.assertEqual((m.msisdn,m.message_text),('123','some text'))

    def test_message_typerror(self):
        self.assertRaises(TypeError,Message,'123')
        self.assertRaises(TypeError,Message,'123',x='some text')

    def test_notification_interface(self):
        self.failUnless(Notification.interface is INotifyMessage)

    def test_notification_normal(self):
        from wasp.status import delivered
        n = Notification('123',delivered,'')
        self.assertEqual((n.message_id,n.status,n.details),('123',delivered,''))

    def test_notification_keyword(self):
        from wasp.status import failed
        n = Notification(details='something bad happened',message_id='123',status=failed)
        self.assertEqual((n.message_id,n.status,n.details),
                         ('123',failed,'something bad happened'))

    def test_notification_typerror(self):
        self.assertRaises(TypeError,Notification,'123')
        self.assertRaises(TypeError,Notification,'123',x='some text')

marker = object()

class TestSendUtility:

    def __call__(self,msisdn,message_text,message_id):
        self.msisdn = msisdn
        self.message_text = message_text
        self.message_id = message_id
        return marker
    
class SendTests(TestCase):

    def setUp(self):
        self.u = TestSendUtility()
        provideUtility(self.u,ISendMessage)

    tearDown = componentTearDown
        
    def test_send_no_utility(self):
        # get rid of any registered stuff:
        componentTearDown()
        # now make sure we get an error if we try and call the receiver
        self.assertRaises(ComponentLookupError,send,'123','message','xyz')

    def test_send(self):
        self.failUnless(send('123','message','xyz') is marker)
        self.assertEqual(self.u.msisdn,'123')
        self.assertEqual(self.u.message_text,'message')
        self.assertEqual(self.u.message_id,'xyz')
        
    def test_send_no_message_id(self):
        self.failUnless(send('123','message') is marker)
        self.assertEqual(self.u.msisdn,'123')
        self.assertEqual(self.u.message_text,'message')
        self.assertEqual(self.u.message_id,None)
        
class Request(dict):

    def __init__(self,body=None,form=None,zope=0):
        dict.__init__(self)
        if form is not None:
            self.update(form)
        self.body = body
        self.zope = zope
        if body and zope==2:
            self['BODY'] = body

    @property
    def bodyStream(self):
        if self.zope==2:
            raise AttributeError
        return StringIO(self.body or '')

    @property
    def form(self):
        return self
    
class TestProcessUtility:

    def __init__(self,notify=None):
        self.notify = notify

    def process(self,request,form,body):
        self.request=request
        self.form=form
        self.body=body
        if self.notify:
            yield self.notify

    def response(self):
        return 'done!'
    
class ReceiverIProcessReponseTests(TestCase):

    def setUp(self):
        self.u = TestProcessUtility()
        provideUtility(self.u,IProcessResponse)

    tearDown = componentTearDown
        
    def test_receiver_noutility(self):
        # get rid of any registered stuff:
        componentTearDown()
        # now make sure we get an error if we try and call the receiver
        self.assertRaises(ComponentLookupError,Receiver(object(),Request()))
        
    def test_Receiver_form(self):
        form = {'x':1,'y':'2'}
        r = Request(form=form)
        self.assertEqual(Receiver(object(),r)(),'done!')
        self.failUnless(self.u.request is r)
        self.assertEqual(self.u.form,form)
        self.assertEqual(self.u.body,'')
        
    def test_Receiver_Zope3_body(self):
        r = Request(body='test',zope=3)
        self.assertEqual(Receiver(object(),r)(),'done!')
        self.failUnless(self.u.request is r)
        self.assertEqual(self.u.form,{})
        self.assertEqual(self.u.body,'test')

    def test_Receiver_Zope2_body(self):
        r = Request(body='test',zope=2)
        self.assertEqual(Receiver(object(),r)(),'done!')
        self.failUnless(self.u.request is r)
        self.assertEqual(self.u.form,{'BODY':'test'})
        self.assertEqual(self.u.body,'test')

class Context: pass

class ReceiverAdapterUtilityTests(TestCase):

    def setUp(self):
        provideUtility(TestProcessUtility(Message('123','message')),IProcessResponse)
        
    tearDown = componentTearDown

    interface = IReceiveMessage
    expected = {'msisdn':'123','message_text':'message'}
    
    def test_nothing(self):
        # now make sure we get an error if there are no message handlers
        # registered
        self.assertRaises(ComponentLookupError,Receiver(object(),Request()))

    def test_adapter(self):
        class Adapter:
            def __init__(self,context):
                Adapter.context = context
            @classmethod
            def __call__(cls,**kw):
                cls.params = kw
        provideAdapter(Adapter,
                       adapts=(Context,),
                       provides=self.interface)
        c = Context()
        Receiver(c,Request())()
        self.assertEqual(Adapter.params,self.expected)
        self.failUnless(Adapter.context is c)
    
    def test_utility(self):
        class Utility:
            def __call__(self,**kw):
                self.params = kw
        u = Utility()
        provideUtility(u,self.interface)
        Receiver(None,Request())()
        self.assertEqual(u.params,self.expected)
    
    def test_both(self):
        class Adapter:
            def __init__(self,context):
                Adapter.context = context
            @classmethod
            def __call__(cls,**kw):
                cls.params = kw
        provideAdapter(Adapter,
                       adapts=(Context,),
                       provides=self.interface)
        class Utility:
            def __call__(self,**kw):
                self.params = kw
        u = Utility()
        provideUtility(u,self.interface)
        c = Context()
        Receiver(c,Request())()
        # the adapter is chosen over the utility
        self.assertEqual(Adapter.params,self.expected)
        self.failUnless(Adapter.context is c)
    
def setUp(test):
    config = getConfigContext()
    def load_zcml(text):
        xmlconfig.string(text,config)
    test.globs['load_zcml']=load_zcml
    browser = Browser()
    browser.handleErrors = False
    test.globs['browser']=browser

FunctionalLayer = ZCMLLayer(
    os.path.join(os.path.split(__file__)[0], 'ftesting.zcml'),
    __name__,
    'wasp Functional Layer',
    allow_teardown=True)

def test_suite():
    
    doc_suite = FunctionalDocFileSuite(
            'readme.txt',
            setUp=setUp,
            tearDown=componentTearDown,
            optionflags=doctest.REPORT_NDIFF|doctest.ELLIPSIS
            )
    doc_suite.layer = FunctionalLayer

    return TestSuite((
        doc_suite,
        doctest.DocTestSuite(),
        makeSuite(Tests),
        makeSuite(SendTests),
        makeSuite(ReceiverIProcessReponseTests),
        makeSuite(ReceiverAdapterUtilityTests),
        ))



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

from wasp.bulksms import SendMessage, ProcessResponse
from wasp.interfaces import ISendMessage,IProcessResponse
from zope.component.zcml import utility
from zope.configuration.fields import Bool
from zope.interface import Interface
from zope.schema import Text

class SenderDirective(Interface):
    """Setup a BulkSMS ISendMessage utility."""

    username = Text(
        title=u"Username",
        description=u"The username sent to BulkSMS",
        )

    password = Text(
        title=u"Password",
        description=u"The password sent to BulkSMS",
        )

    secure = Bool(
        title=u"Use secure connection",
        description=u"Configures whether an HTTP or HTTPS connection is used when contacting BulkSMS",
        required=False
        )

    want_report = Bool(
        title=u"Ask BulkSMS to send reports",
        description=u"Configures whether or not reports will be requested when sending messages",
        required=False
        )

class ProcessorDirective(Interface):
    """Setup a BulkSMS IProcessResponse utility."""

    incoming_password = Text(
        title=u"Password for BulkSMS requests",
        description=u"Configures whether the SendMessage utility will return True or False when called",
        )

def setupISendMessageUtility(_context,username,password,**kw):
    u = SendMessage(username,password,**kw)
    utility(_context, ISendMessage, u)

def setupIProcessResponseUtility(_context,incoming_password):
    u = ProcessResponse(incoming_password)
    utility(_context, IProcessResponse, u)



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

from interfaces import ISendMessage,IReceiveMessage,INotifyMessage,IProcessResponse
from zope.component import getUtility,queryAdapter
from zope.publisher.browser import BrowserView

def send(msisdn,message_text,message_id=None):
    return getUtility(ISendMessage)(msisdn,message_text,message_id)

class Receiver(BrowserView):

    def __call__(self):
        # we have to use duck typing here because Zope 2
        # requests lie about implementing interfaces :-(
        bodyStream = getattr(self.request,'bodyStream',None)
        if bodyStream is None:
            # zope 2
            body = self.request.get('BODY','')
        else:
            # zope 3
            body = self.request.bodyStream.read()
        # despatch to IProcessResponse utility and handle returned events
        utility = getUtility(IProcessResponse)
        for event in utility.process(self.request,self.request.form,body):
            adapter = queryAdapter(self.context,event.interface)
            if adapter is not None:
                adapter(**event.__dict__)
            else:
                getUtility(event.interface)(**event.__dict__)
        return utility.response()

class Notification:
    """
    A notification object used to dispatch notifications from an
    IProcessResponse utility to INotifyMessage utilities or adapters.

    message_id - the message_id passed to the `send` method

    status - one of the statuses in wasp.status

    details - a string giving more information about why the status
              was returned. In the case of errors, this should contain
              the full error message returned from the WASP.
              If the status is wasp.status.Delivered then this may be
              an empty string.
    """
                          
    interface = INotifyMessage
                          
    def __init__(self,message_id,status,details):
        self.message_id = message_id
        self.status = status
        self.details = details

class Message:
    """
    A message object used to dispatch messages recieved by an
    IProcessResponse utility to IRecieveMessage utilities or adapters.

    msisdn - the msisdn the message was recieved to

    message_text - the text of the recieved message.
    """

    interface = IReceiveMessage
                          
    def __init__(self,msisdn,message_text):
        self.msisdn = msisdn
        self.message_text = message_text

class SendException(Exception):
    """
    An exception to raise from within an ISendMessage implementation's
    __call__ method when something goes wrong.
    """



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.
"""
This module contains constants for statuses returned in notifications
from WASPs about the delivery of messages sent wit hteh `send`
function.

They are implemented as classes in case they need to be adapted using
the component architecture at a later date.
"""

class Status:
    pass

class Delivered(Status):
    """
    The message was successfully delivered to its recipient.
    """

class Failed(Status):
    """
    The message delivery has failed.
    """

delivered = Delivered()
failed = Failed()



# Copyright (c) 2008 Simplistix Ltd
#
# All rights reserved.

import os
from setuptools import setup, find_packages

this_dir = os.path.join(os.path.dirname(__file__),'wasp')

setup(
    name='wasp',
    version=file(os.path.join(this_dir,'version.txt')).read().strip(),
    author='Chris Withers',
    author_email='chris@simplistix.co.uk',
    license='All rights reserved',
    description="A library for abstracting interactions with different WASPs",
    long_description=open(os.path.join(this_dir,'readme.txt')).read(),
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
    'zope.component',
    'zope.configuration',
    'zope.interface',
    'zope.publisher',
    'zope.security',
    ],
    )

# to build the egg, do:
# python setup.py sdist bdist_egg
# ...or...
#  bin/buildout setup setup.py sdist bdist_egg



# Copyright (c) 2008 Simplistix Ltd
#
# This Software is released under the MIT License:
# http://www.opensource.org/licenses/mit-license.html
# See license.txt for more details.

from zope.interface import Interface

class IReceiveMessage(Interface):

    def __call__(msisdn,message_text):
        """
        This method is called to handle incoming messages.
        The implementation should do whatever is required by the
        application.

        msisdn - the msisdn that sent the incoming message

        message_text - the text of the message
        """

class INotifyMessage(Interface):

    def __call__(message_id,status,details):
        """
        This method is called when status notifications about a
        previously sent message are recieved.

        message_id - will be that passed in the call to the `send`
                     method

        status - will be the 
        """

# You only need to know details of the following interfaces if you're
# implementing a particular WASP.

class ISendMessage(Interface):

    def __call__(msisdn,message_text,message_id=None):
        """
        Send the message_text to the specified msisdn.
        The message_id is a unique id for this message. If supplied,
        it will be returned to any configured INotifyMessage.

        This returns either:

        True - indicating that the message has been
               successfully delivered

        False - indicating that a notification of delivery status will
                be sent.
                (NB: an IRecieveMessage must be configured in this
                     case)

        In other cases, an exception should be raised indicating the
        nature of the problem that occurred.
        """

class IProcessResponse(Interface):
    """
    Each WASP implementation should provide a utility implementing
    this interface.
    """

    def process(request,form,body):
        """
        This method is called by the reciever view with information
        from the request received. This should be parsed and an
        iterable returned that yields wasp.Notification and
        wasp.Message objects. For more information on these see their
        docstrings and code.
        It's acceptable to return an empty sequence or a sequence
        containing one item.
        """

    def response():
        """
        This should return a string containing the values to be
        returned by the Receiver view.
        This will need to be done in the event that the WASP
        implementation requires a specific response to any HTTP POSTs
        or GETs that it sends.
        """
        
